"use client";
import "./RatingSection.scss";
import Image from "next/image";
import { useEffect, useState } from "react";
import { useRouter } from "next/navigation";
import SearchForm from "@/features/SearchForm/SearchForm";
import arrow_down_icon from "./icons/arrow-down.svg";
import arrow_up_icon from "./icons/arrow-up.svg";
import like_icon from "./icons/like.svg";
import message_icon from "./icons/message.svg";
import { useRatingStore } from "./ratingSectionStore";
import { useShallow } from "zustand/react/shallow";
import Link from "next/link";
import {
  ROAD_TYPES,
  ROAD_TYPES_COLORS,
} from "@/shared/variables/road-types";
import RoadType from "@/entities/RoadType/RoadType";
import Typography from "@/shared/ui/components/Typography/Typography";
import Paragraph from "@/shared/ui/components/Paragraph/Paragraph";
import arrows from "../../shared/icons/arrows.svg";

interface IRatingSectionProps {
  [key: string]: string;
}

const RatingSection: React.FC<IRatingSectionProps> = ({
  categories = "1,2,3,4,5,6",
  queryMap,
  queryRating,
}: IRatingSectionProps) => {
  const [ratingSearch, setRatingSearch] = useState<string>(
    queryRating || ""
  );
  const router = useRouter();

  const reports = useRatingStore(useShallow((state) => state.data));
  const getReports = useRatingStore(
    useShallow((state) => state.getReports)
  );

  useEffect(() => {
    getReports(ratingSearch);
  }, []);

  const handleSubmit: React.FormEventHandler<
    HTMLFormElement
  > = async (e) => {
    e.preventDefault();
    const formData = new FormData(e.currentTarget);
    setRatingSearch(formData.get("rating-search") as string);

    router.push(
      `/?тип-дороги=${categories}${
        queryMap ? `&поиск-на-карте=${queryMap}` : ""
      }${ratingSearch ? `&поиск-рейтинг=${ratingSearch}` : ""}`,
      {
        scroll: false,
      }
    );

    getReports(ratingSearch);
  };

  const sliceDate = (date: string) => {
    return `${date.slice(8, 10)}.${date.slice(5, 7)}.${date.slice(
      0,
      4
    )}`;
  };

  const sliceLocation = (location: string) => {
    if (location.length > 15) return `${location.slice(0, 15)}...`;

    return location;
  };

  const sliceDescription = (description: string) => {
    if (description.length > 49) {
      return `${description.slice(0, 50)}...`;
    }

    return description;
  };

  const params = [
    {
      id: 1,
      name: "Дата",
      handleClick() {
        console.log(this.name);
      },
    },
    { id: 2, name: "Адрес" },
    { id: 3, name: "Статус" },
    { id: 4, name: "Описание" },
    {
      id: 5,
      name: "Комментарии",
      handleClick() {
        console.log(this.name);
      },
    },
    {
      id: 6,
      name: "Рейтинг",
      handleClick() {
        console.log(this.name);
      },
    },
  ];

  return (
    <section className="rating-section">
      <div className="rating-section__header">
        <Typography element="h3">Рейтинг</Typography>
        <Paragraph>
          Обсуждаем дороги: рейтинг, опыт, комфорт в пути!
        </Paragraph>
      </div>
      <SearchForm
        onChange={(e) => setRatingSearch(e.target.value)}
        name="rating-search"
        placeholder="Введите адрес"
        value={ratingSearch}
        handleSubmit={handleSubmit}
      />
      <div className="rating-section__table">
        <table>
          <thead>
            <tr>
              {params.map((p) => (
                <td key={p.id}>
                  {p.handleClick ? (
                    <button onClick={p.handleClick}>
                      {p.name}
                      <Image src={arrows} alt="Up and Down Arrows" />
                    </button>
                  ) : (
                    p.name
                  )}
                </td>
              ))}
            </tr>
          </thead>
          <tbody>
            {reports.results.map((report) => (
              <tr key={report.id}>
                <td id="rating-section-date">
                  {sliceDate(report.created_at)}
                </td>
                <td id="rating-section-link">
                  <Link href={`/report/${report.id}`}>
                    {sliceLocation(report.location[0].address)}
                  </Link>
                </td>
                <td>
                  <RoadType
                    color={ROAD_TYPES_COLORS[report.category]}
                  >
                    {ROAD_TYPES[report.category]}
                  </RoadType>
                </td>
                <td id="rating-section-description">
                  {sliceDescription(report.description)}
                </td>
                <td id="rating-section-reviews">
                  <Image src={message_icon} alt="Message Icon" />
                  {report.count_reviews}
                </td>
                <td id="rating-section-likes">
                  <Image src={like_icon} alt="Like Icon" />
                  {report.total_likes}
                </td>
              </tr>
            ))}
          </tbody>
        </table>
      </div>
    </section>
  );
};

export default RatingSection;
